#include <gtkmm/box.h>
#include <gtkmm/liststore.h>
#include "remote.h"
#include "trainpanel.h"

using namespace std;

TrainPanel::TrainPanel(Remote &r, R2C2::Client &c, R2C2::NetTrain &t):
	remote(r),
	client(c),
	train(t)
{
	train.signal_name_changed.connect(sigc::mem_fun(this, &TrainPanel::name_changed));
	train.signal_control_changed.connect(sigc::mem_fun(this, &TrainPanel::control_changed));
	train.signal_function_changed.connect(sigc::mem_fun(this, &TrainPanel::function_changed));
	train.signal_route_changed.connect(sigc::mem_fun(this, &TrainPanel::route_changed));
	train.signal_status_changed.connect(sigc::mem_fun(this, &TrainPanel::status_changed));

	set_label(train.get_name());

	Gtk::VBox *vbox = new Gtk::VBox(false, 5);
	add(*manage(vbox));
	vbox->set_border_width(5);

	Gtk::HBox *hbox = new Gtk::HBox(false, 5);
	vbox->add(*manage(hbox));

	hbox->pack_start(*manage(scl_speed = new Gtk::HScale), true, true);
	scl_speed->set_digits(0);
	scl_speed->set_range(0, 200);
	scl_speed->set_increments(5, 5);
	scl_speed->set_size_request(210, -1);
	scl_speed->signal_value_changed().connect(sigc::mem_fun(this, &TrainPanel::ui_speed_changed));

	hbox->pack_start(*manage(chk_reverse = new Gtk::CheckButton("Rev")), false, true);
	chk_reverse->signal_toggled().connect(sigc::mem_fun(this, &TrainPanel::ui_reverse_changed));

	Gtk::HBox *func_box = new Gtk::HBox(false, 5);
	vbox->add(*manage(func_box));
	const R2C2::VehicleType::FunctionMap &funcs = train.get_loco_type().get_functions();
	for(R2C2::VehicleType::FunctionMap::const_iterator i=funcs.begin(); i!=funcs.end(); ++i)
	{
		Gtk::CheckButton *&chk = chk_funcs[i->first];
		chk = new Gtk::CheckButton(i->second);
		func_box->pack_start(*manage(chk), false, true);
		chk->signal_toggled().connect(sigc::bind(sigc::mem_fun(this, &TrainPanel::ui_function_changed), i->first));
	}

	Glib::RefPtr<Gtk::ListStore> route_store = Gtk::ListStore::create(route_columns);
	vbox->add(*manage(cmb_route = new Gtk::ComboBox(static_cast<const Glib::RefPtr<Gtk::TreeModel> &>(route_store))));
	cmb_route->pack_start(route_columns.name);
	route_store->append();
	const list<string> &routes = client.get_routes();
	for(list<string>::const_iterator i=routes.begin(); i!=routes.end(); ++i)
	{
		Gtk::TreeIter iter = route_store->append();
		(*iter)[route_columns.name] = *i;
	}
	cmb_route->signal_changed().connect(sigc::mem_fun(this, &TrainPanel::ui_route_changed));

	vbox->add(*manage(lbl_status = new Gtk::Label));

	show_all();
}

void TrainPanel::name_changed(const string &name)
{
	set_label(name);
}

void TrainPanel::status_changed(const string &status)
{
	lbl_status->set_text(status);
}

void TrainPanel::control_changed(const string &control, float value)
{
	if(control=="speed")
	{
		// XXX It would be better to make the VehicleType give us the catalogue
		scl_speed->set_value(value*3.6/remote.get_catalogue().get_scale());
	}
	else if(control=="reverse")
		chk_reverse->set_active(value);
}

void TrainPanel::function_changed(unsigned func, bool set)
{
	std::map<unsigned, Gtk::CheckButton *>::iterator i = chk_funcs.find(func);
	if(i!=chk_funcs.end())
		i->second->set_active(set);
}

void TrainPanel::route_changed(const string &route)
{
	Gtk::TreeNodeChildren children = cmb_route->get_model()->children();
	for(Gtk::TreeIter i=children.begin(); i!=children.end(); ++i)
		if((*i)[route_columns.name]==route)
		{
			cmb_route->set_active(i);
			break;
		}
}

void TrainPanel::ui_speed_changed()
{
	float speed = scl_speed->get_value()/3.6*remote.get_catalogue().get_scale();
	train.set_control("speed", speed);
}

void TrainPanel::ui_reverse_changed()
{
	if(train.get_control("speed"))
	{
		train.set_control("speed", 0);
		chk_reverse->set_active(train.get_control("reverse"));
	}
	else
		train.set_control("reverse", chk_reverse->get_active());
}

void TrainPanel::ui_function_changed(unsigned func)
{
	std::map<unsigned, Gtk::CheckButton *>::iterator i = chk_funcs.find(func);
	if(i!=chk_funcs.end())
		train.set_function(func, i->second->get_active());
}

void TrainPanel::ui_route_changed()
{
	Gtk::TreeIter iter = cmb_route->get_active();
	train.set_route(Glib::ustring((*iter)[route_columns.name]));
}


TrainPanel::RouteRecord::RouteRecord()
{
	add(name);
}
